/****************************************************************************
** Meta object code from reading C++ file 'qmediaplaylist.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/multimedia/playback/qmediaplaylist.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qmediaplaylist.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QMediaPlaylist_t {
    QByteArrayData data[39];
    char stringdata0[507];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QMediaPlaylist_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QMediaPlaylist_t qt_meta_stringdata_QMediaPlaylist = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QMediaPlaylist"
QT_MOC_LITERAL(1, 15, 19), // "currentIndexChanged"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 5), // "index"
QT_MOC_LITERAL(4, 42, 19), // "playbackModeChanged"
QT_MOC_LITERAL(5, 62, 28), // "QMediaPlaylist::PlaybackMode"
QT_MOC_LITERAL(6, 91, 4), // "mode"
QT_MOC_LITERAL(7, 96, 19), // "currentMediaChanged"
QT_MOC_LITERAL(8, 116, 13), // "QMediaContent"
QT_MOC_LITERAL(9, 130, 22), // "mediaAboutToBeInserted"
QT_MOC_LITERAL(10, 153, 5), // "start"
QT_MOC_LITERAL(11, 159, 3), // "end"
QT_MOC_LITERAL(12, 163, 13), // "mediaInserted"
QT_MOC_LITERAL(13, 177, 21), // "mediaAboutToBeRemoved"
QT_MOC_LITERAL(14, 199, 12), // "mediaRemoved"
QT_MOC_LITERAL(15, 212, 12), // "mediaChanged"
QT_MOC_LITERAL(16, 225, 6), // "loaded"
QT_MOC_LITERAL(17, 232, 10), // "loadFailed"
QT_MOC_LITERAL(18, 243, 7), // "shuffle"
QT_MOC_LITERAL(19, 251, 4), // "next"
QT_MOC_LITERAL(20, 256, 8), // "previous"
QT_MOC_LITERAL(21, 265, 15), // "setCurrentIndex"
QT_MOC_LITERAL(22, 281, 13), // "_q_loadFailed"
QT_MOC_LITERAL(23, 295, 21), // "QMediaPlaylist::Error"
QT_MOC_LITERAL(24, 317, 12), // "playbackMode"
QT_MOC_LITERAL(25, 330, 12), // "currentMedia"
QT_MOC_LITERAL(26, 343, 12), // "currentIndex"
QT_MOC_LITERAL(27, 356, 12), // "PlaybackMode"
QT_MOC_LITERAL(28, 369, 15), // "CurrentItemOnce"
QT_MOC_LITERAL(29, 385, 17), // "CurrentItemInLoop"
QT_MOC_LITERAL(30, 403, 10), // "Sequential"
QT_MOC_LITERAL(31, 414, 4), // "Loop"
QT_MOC_LITERAL(32, 419, 6), // "Random"
QT_MOC_LITERAL(33, 426, 5), // "Error"
QT_MOC_LITERAL(34, 432, 7), // "NoError"
QT_MOC_LITERAL(35, 440, 11), // "FormatError"
QT_MOC_LITERAL(36, 452, 23), // "FormatNotSupportedError"
QT_MOC_LITERAL(37, 476, 12), // "NetworkError"
QT_MOC_LITERAL(38, 489, 17) // "AccessDeniedError"

    },
    "QMediaPlaylist\0currentIndexChanged\0\0"
    "index\0playbackModeChanged\0"
    "QMediaPlaylist::PlaybackMode\0mode\0"
    "currentMediaChanged\0QMediaContent\0"
    "mediaAboutToBeInserted\0start\0end\0"
    "mediaInserted\0mediaAboutToBeRemoved\0"
    "mediaRemoved\0mediaChanged\0loaded\0"
    "loadFailed\0shuffle\0next\0previous\0"
    "setCurrentIndex\0_q_loadFailed\0"
    "QMediaPlaylist::Error\0playbackMode\0"
    "currentMedia\0currentIndex\0PlaybackMode\0"
    "CurrentItemOnce\0CurrentItemInLoop\0"
    "Sequential\0Loop\0Random\0Error\0NoError\0"
    "FormatError\0FormatNotSupportedError\0"
    "NetworkError\0AccessDeniedError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QMediaPlaylist[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       3,  136, // properties
       2,  148, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       4,    1,   92,    2, 0x06 /* Public */,
       7,    1,   95,    2, 0x06 /* Public */,
       9,    2,   98,    2, 0x06 /* Public */,
      12,    2,  103,    2, 0x06 /* Public */,
      13,    2,  108,    2, 0x06 /* Public */,
      14,    2,  113,    2, 0x06 /* Public */,
      15,    2,  118,    2, 0x06 /* Public */,
      16,    0,  123,    2, 0x06 /* Public */,
      17,    0,  124,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    0,  125,    2, 0x0a /* Public */,
      19,    0,  126,    2, 0x0a /* Public */,
      20,    0,  127,    2, 0x0a /* Public */,
      21,    1,  128,    2, 0x0a /* Public */,
      22,    2,  131,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   10,   11,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   10,   11,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   10,   11,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   10,   11,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   10,   11,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, 0x80000000 | 23, QMetaType::QString,    2,    2,

 // properties: name, type, flags
      24, 0x80000000 | 5, 0x0049510b,
      25, 0x80000000 | 8, 0x00495009,
      26, QMetaType::Int, 0x00495103,

 // properties: notify_signal_id
       1,
       2,
       0,

 // enums: name, alias, flags, count, data
      27,   27, 0x0,    5,  158,
      33,   33, 0x0,    5,  168,

 // enum data: key, value
      28, uint(QMediaPlaylist::CurrentItemOnce),
      29, uint(QMediaPlaylist::CurrentItemInLoop),
      30, uint(QMediaPlaylist::Sequential),
      31, uint(QMediaPlaylist::Loop),
      32, uint(QMediaPlaylist::Random),
      34, uint(QMediaPlaylist::NoError),
      35, uint(QMediaPlaylist::FormatError),
      36, uint(QMediaPlaylist::FormatNotSupportedError),
      37, uint(QMediaPlaylist::NetworkError),
      38, uint(QMediaPlaylist::AccessDeniedError),

       0        // eod
};

void QMediaPlaylist::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QMediaPlaylist *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->playbackModeChanged((*reinterpret_cast< QMediaPlaylist::PlaybackMode(*)>(_a[1]))); break;
        case 2: _t->currentMediaChanged((*reinterpret_cast< const QMediaContent(*)>(_a[1]))); break;
        case 3: _t->mediaAboutToBeInserted((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->mediaInserted((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->mediaAboutToBeRemoved((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->mediaRemoved((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->mediaChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 8: _t->loaded(); break;
        case 9: _t->loadFailed(); break;
        case 10: _t->shuffle(); break;
        case 11: _t->next(); break;
        case 12: _t->previous(); break;
        case 13: _t->setCurrentIndex((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->d_func()->_q_loadFailed((*reinterpret_cast< QMediaPlaylist::Error(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaPlaylist::PlaybackMode >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaContent >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaPlaylist::Error >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QMediaPlaylist::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QMediaPlaylist::currentIndexChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QMediaPlaylist::*)(QMediaPlaylist::PlaybackMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QMediaPlaylist::playbackModeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QMediaPlaylist::*)(const QMediaContent & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QMediaPlaylist::currentMediaChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QMediaPlaylist::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QMediaPlaylist::mediaAboutToBeInserted)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QMediaPlaylist::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QMediaPlaylist::mediaInserted)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QMediaPlaylist::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QMediaPlaylist::mediaAboutToBeRemoved)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QMediaPlaylist::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QMediaPlaylist::mediaRemoved)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QMediaPlaylist::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QMediaPlaylist::mediaChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QMediaPlaylist::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QMediaPlaylist::loaded)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QMediaPlaylist::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QMediaPlaylist::loadFailed)) {
                *result = 9;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaContent >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaPlaylist::PlaybackMode >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QMediaPlaylist *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QMediaPlaylist::PlaybackMode*>(_v) = _t->playbackMode(); break;
        case 1: *reinterpret_cast< QMediaContent*>(_v) = _t->currentMedia(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->currentIndex(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QMediaPlaylist *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPlaybackMode(*reinterpret_cast< QMediaPlaylist::PlaybackMode*>(_v)); break;
        case 2: _t->setCurrentIndex(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QMediaPlaylist::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QMediaPlaylist.data,
    qt_meta_data_QMediaPlaylist,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QMediaPlaylist::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QMediaPlaylist::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QMediaPlaylist.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QMediaBindableInterface"))
        return static_cast< QMediaBindableInterface*>(this);
    if (!strcmp(_clname, "org.qt-project.qt.mediabindable/5.0"))
        return static_cast< QMediaBindableInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int QMediaPlaylist::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QMediaPlaylist::currentIndexChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QMediaPlaylist::playbackModeChanged(QMediaPlaylist::PlaybackMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QMediaPlaylist::currentMediaChanged(const QMediaContent & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QMediaPlaylist::mediaAboutToBeInserted(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QMediaPlaylist::mediaInserted(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QMediaPlaylist::mediaAboutToBeRemoved(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QMediaPlaylist::mediaRemoved(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QMediaPlaylist::mediaChanged(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QMediaPlaylist::loaded()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QMediaPlaylist::loadFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
