/****************************************************************************
** Meta object code from reading C++ file 'qmedianetworkplaylistprovider_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/multimedia/playback/qmedianetworkplaylistprovider_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qmedianetworkplaylistprovider_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QMediaNetworkPlaylistProvider_t {
    QByteArrayData data[8];
    char stringdata0[122];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QMediaNetworkPlaylistProvider_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QMediaNetworkPlaylistProvider_t qt_meta_stringdata_QMediaNetworkPlaylistProvider = {
    {
QT_MOC_LITERAL(0, 0, 29), // "QMediaNetworkPlaylistProvider"
QT_MOC_LITERAL(1, 30, 7), // "shuffle"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 20), // "_q_handleParserError"
QT_MOC_LITERAL(4, 60, 32), // "QPlaylistFileParser::ParserError"
QT_MOC_LITERAL(5, 93, 3), // "err"
QT_MOC_LITERAL(6, 97, 16), // "_q_handleNewItem"
QT_MOC_LITERAL(7, 114, 7) // "content"

    },
    "QMediaNetworkPlaylistProvider\0shuffle\0"
    "\0_q_handleParserError\0"
    "QPlaylistFileParser::ParserError\0err\0"
    "_q_handleNewItem\0content"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QMediaNetworkPlaylistProvider[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x0a /* Public */,
       3,    2,   30,    2, 0x08 /* Private */,
       6,    1,   35,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4, QMetaType::QString,    5,    2,
    QMetaType::Void, QMetaType::QVariant,    7,

       0        // eod
};

void QMediaNetworkPlaylistProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QMediaNetworkPlaylistProvider *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->shuffle(); break;
        case 1: _t->d_func()->_q_handleParserError((*reinterpret_cast< QPlaylistFileParser::ParserError(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 2: _t->d_func()->_q_handleNewItem((*reinterpret_cast< const QVariant(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QMediaNetworkPlaylistProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QMediaPlaylistProvider::staticMetaObject>(),
    qt_meta_stringdata_QMediaNetworkPlaylistProvider.data,
    qt_meta_data_QMediaNetworkPlaylistProvider,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QMediaNetworkPlaylistProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QMediaNetworkPlaylistProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QMediaNetworkPlaylistProvider.stringdata0))
        return static_cast<void*>(this);
    return QMediaPlaylistProvider::qt_metacast(_clname);
}

int QMediaNetworkPlaylistProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMediaPlaylistProvider::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
