/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativecamerarecorder_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/multimedia/qdeclarativecamerarecorder_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativecamerarecorder_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QDeclarativeCameraRecorder_t {
    QByteArrayData data[95];
    char stringdata0[1475];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeCameraRecorder_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeCameraRecorder_t qt_meta_stringdata_QDeclarativeCameraRecorder = {
    {
QT_MOC_LITERAL(0, 0, 26), // "QDeclarativeCameraRecorder"
QT_MOC_LITERAL(1, 27, 20), // "recorderStateChanged"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 41), // "QDeclarativeCameraRecorder::R..."
QT_MOC_LITERAL(4, 91, 5), // "state"
QT_MOC_LITERAL(5, 97, 21), // "recorderStatusChanged"
QT_MOC_LITERAL(6, 119, 15), // "durationChanged"
QT_MOC_LITERAL(7, 135, 8), // "duration"
QT_MOC_LITERAL(8, 144, 12), // "mutedChanged"
QT_MOC_LITERAL(9, 157, 5), // "muted"
QT_MOC_LITERAL(10, 163, 21), // "outputLocationChanged"
QT_MOC_LITERAL(11, 185, 8), // "location"
QT_MOC_LITERAL(12, 194, 21), // "actualLocationChanged"
QT_MOC_LITERAL(13, 216, 5), // "error"
QT_MOC_LITERAL(14, 222, 33), // "QDeclarativeCameraRecorder::E..."
QT_MOC_LITERAL(15, 256, 9), // "errorCode"
QT_MOC_LITERAL(16, 266, 11), // "errorString"
QT_MOC_LITERAL(17, 278, 15), // "metaDataChanged"
QT_MOC_LITERAL(18, 294, 3), // "key"
QT_MOC_LITERAL(19, 298, 5), // "value"
QT_MOC_LITERAL(20, 304, 24), // "captureResolutionChanged"
QT_MOC_LITERAL(21, 329, 17), // "audioCodecChanged"
QT_MOC_LITERAL(22, 347, 5), // "codec"
QT_MOC_LITERAL(23, 353, 17), // "videoCodecChanged"
QT_MOC_LITERAL(24, 371, 21), // "mediaContainerChanged"
QT_MOC_LITERAL(25, 393, 9), // "container"
QT_MOC_LITERAL(26, 403, 16), // "frameRateChanged"
QT_MOC_LITERAL(27, 420, 3), // "arg"
QT_MOC_LITERAL(28, 424, 19), // "videoBitRateChanged"
QT_MOC_LITERAL(29, 444, 19), // "audioBitRateChanged"
QT_MOC_LITERAL(30, 464, 20), // "audioChannelsChanged"
QT_MOC_LITERAL(31, 485, 22), // "audioSampleRateChanged"
QT_MOC_LITERAL(32, 508, 24), // "audioEncodingModeChanged"
QT_MOC_LITERAL(33, 533, 12), // "EncodingMode"
QT_MOC_LITERAL(34, 546, 12), // "encodingMode"
QT_MOC_LITERAL(35, 559, 24), // "videoEncodingModeChanged"
QT_MOC_LITERAL(36, 584, 17), // "setOutputLocation"
QT_MOC_LITERAL(37, 602, 6), // "record"
QT_MOC_LITERAL(38, 609, 4), // "stop"
QT_MOC_LITERAL(39, 614, 16), // "setRecorderState"
QT_MOC_LITERAL(40, 631, 8), // "setMuted"
QT_MOC_LITERAL(41, 640, 11), // "setMetadata"
QT_MOC_LITERAL(42, 652, 20), // "setCaptureResolution"
QT_MOC_LITERAL(43, 673, 10), // "resolution"
QT_MOC_LITERAL(44, 684, 13), // "setAudioCodec"
QT_MOC_LITERAL(45, 698, 13), // "setVideoCodec"
QT_MOC_LITERAL(46, 712, 17), // "setMediaContainer"
QT_MOC_LITERAL(47, 730, 12), // "setFrameRate"
QT_MOC_LITERAL(48, 743, 9), // "frameRate"
QT_MOC_LITERAL(49, 753, 15), // "setVideoBitRate"
QT_MOC_LITERAL(50, 769, 4), // "rate"
QT_MOC_LITERAL(51, 774, 15), // "setAudioBitRate"
QT_MOC_LITERAL(52, 790, 16), // "setAudioChannels"
QT_MOC_LITERAL(53, 807, 8), // "channels"
QT_MOC_LITERAL(54, 816, 18), // "setAudioSampleRate"
QT_MOC_LITERAL(55, 835, 20), // "setVideoEncodingMode"
QT_MOC_LITERAL(56, 856, 20), // "setAudioEncodingMode"
QT_MOC_LITERAL(57, 877, 19), // "updateRecorderState"
QT_MOC_LITERAL(58, 897, 21), // "QMediaRecorder::State"
QT_MOC_LITERAL(59, 919, 19), // "updateRecorderError"
QT_MOC_LITERAL(60, 939, 21), // "QMediaRecorder::Error"
QT_MOC_LITERAL(61, 961, 20), // "updateActualLocation"
QT_MOC_LITERAL(62, 982, 13), // "recorderState"
QT_MOC_LITERAL(63, 996, 13), // "RecorderState"
QT_MOC_LITERAL(64, 1010, 14), // "recorderStatus"
QT_MOC_LITERAL(65, 1025, 14), // "RecorderStatus"
QT_MOC_LITERAL(66, 1040, 10), // "videoCodec"
QT_MOC_LITERAL(67, 1051, 12), // "videoBitRate"
QT_MOC_LITERAL(68, 1064, 17), // "videoEncodingMode"
QT_MOC_LITERAL(69, 1082, 10), // "audioCodec"
QT_MOC_LITERAL(70, 1093, 12), // "audioBitRate"
QT_MOC_LITERAL(71, 1106, 13), // "audioChannels"
QT_MOC_LITERAL(72, 1120, 15), // "audioSampleRate"
QT_MOC_LITERAL(73, 1136, 17), // "audioEncodingMode"
QT_MOC_LITERAL(74, 1154, 14), // "mediaContainer"
QT_MOC_LITERAL(75, 1169, 14), // "outputLocation"
QT_MOC_LITERAL(76, 1184, 14), // "actualLocation"
QT_MOC_LITERAL(77, 1199, 5), // "Error"
QT_MOC_LITERAL(78, 1205, 12), // "StoppedState"
QT_MOC_LITERAL(79, 1218, 14), // "RecordingState"
QT_MOC_LITERAL(80, 1233, 17), // "UnavailableStatus"
QT_MOC_LITERAL(81, 1251, 14), // "UnloadedStatus"
QT_MOC_LITERAL(82, 1266, 13), // "LoadingStatus"
QT_MOC_LITERAL(83, 1280, 12), // "LoadedStatus"
QT_MOC_LITERAL(84, 1293, 14), // "StartingStatus"
QT_MOC_LITERAL(85, 1308, 15), // "RecordingStatus"
QT_MOC_LITERAL(86, 1324, 12), // "PausedStatus"
QT_MOC_LITERAL(87, 1337, 16), // "FinalizingStatus"
QT_MOC_LITERAL(88, 1354, 23), // "ConstantQualityEncoding"
QT_MOC_LITERAL(89, 1378, 23), // "ConstantBitRateEncoding"
QT_MOC_LITERAL(90, 1402, 22), // "AverageBitRateEncoding"
QT_MOC_LITERAL(91, 1425, 7), // "NoError"
QT_MOC_LITERAL(92, 1433, 13), // "ResourceError"
QT_MOC_LITERAL(93, 1447, 11), // "FormatError"
QT_MOC_LITERAL(94, 1459, 15) // "OutOfSpaceError"

    },
    "QDeclarativeCameraRecorder\0"
    "recorderStateChanged\0\0"
    "QDeclarativeCameraRecorder::RecorderState\0"
    "state\0recorderStatusChanged\0durationChanged\0"
    "duration\0mutedChanged\0muted\0"
    "outputLocationChanged\0location\0"
    "actualLocationChanged\0error\0"
    "QDeclarativeCameraRecorder::Error\0"
    "errorCode\0errorString\0metaDataChanged\0"
    "key\0value\0captureResolutionChanged\0"
    "audioCodecChanged\0codec\0videoCodecChanged\0"
    "mediaContainerChanged\0container\0"
    "frameRateChanged\0arg\0videoBitRateChanged\0"
    "audioBitRateChanged\0audioChannelsChanged\0"
    "audioSampleRateChanged\0audioEncodingModeChanged\0"
    "EncodingMode\0encodingMode\0"
    "videoEncodingModeChanged\0setOutputLocation\0"
    "record\0stop\0setRecorderState\0setMuted\0"
    "setMetadata\0setCaptureResolution\0"
    "resolution\0setAudioCodec\0setVideoCodec\0"
    "setMediaContainer\0setFrameRate\0frameRate\0"
    "setVideoBitRate\0rate\0setAudioBitRate\0"
    "setAudioChannels\0channels\0setAudioSampleRate\0"
    "setVideoEncodingMode\0setAudioEncodingMode\0"
    "updateRecorderState\0QMediaRecorder::State\0"
    "updateRecorderError\0QMediaRecorder::Error\0"
    "updateActualLocation\0recorderState\0"
    "RecorderState\0recorderStatus\0"
    "RecorderStatus\0videoCodec\0videoBitRate\0"
    "videoEncodingMode\0audioCodec\0audioBitRate\0"
    "audioChannels\0audioSampleRate\0"
    "audioEncodingMode\0mediaContainer\0"
    "outputLocation\0actualLocation\0Error\0"
    "StoppedState\0RecordingState\0"
    "UnavailableStatus\0UnloadedStatus\0"
    "LoadingStatus\0LoadedStatus\0StartingStatus\0"
    "RecordingStatus\0PausedStatus\0"
    "FinalizingStatus\0ConstantQualityEncoding\0"
    "ConstantBitRateEncoding\0AverageBitRateEncoding\0"
    "NoError\0ResourceError\0FormatError\0"
    "OutOfSpaceError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeCameraRecorder[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      39,   14, // methods
      19,  326, // properties
       4,  402, // enums/sets
       0,    0, // constructors
       0,       // flags
      19,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  209,    2, 0x06 /* Public */,
       5,    0,  212,    2, 0x06 /* Public */,
       6,    1,  213,    2, 0x06 /* Public */,
       8,    1,  216,    2, 0x06 /* Public */,
      10,    1,  219,    2, 0x06 /* Public */,
      12,    1,  222,    2, 0x06 /* Public */,
      13,    2,  225,    2, 0x06 /* Public */,
      17,    2,  230,    2, 0x06 /* Public */,
      20,    1,  235,    2, 0x06 /* Public */,
      21,    1,  238,    2, 0x06 /* Public */,
      23,    1,  241,    2, 0x06 /* Public */,
      24,    1,  244,    2, 0x06 /* Public */,
      26,    1,  247,    2, 0x06 /* Public */,
      28,    1,  250,    2, 0x06 /* Public */,
      29,    1,  253,    2, 0x06 /* Public */,
      30,    1,  256,    2, 0x06 /* Public */,
      31,    1,  259,    2, 0x06 /* Public */,
      32,    1,  262,    2, 0x06 /* Public */,
      35,    1,  265,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      36,    1,  268,    2, 0x0a /* Public */,
      37,    0,  271,    2, 0x0a /* Public */,
      38,    0,  272,    2, 0x0a /* Public */,
      39,    1,  273,    2, 0x0a /* Public */,
      40,    1,  276,    2, 0x0a /* Public */,
      41,    2,  279,    2, 0x0a /* Public */,
      42,    1,  284,    2, 0x0a /* Public */,
      44,    1,  287,    2, 0x0a /* Public */,
      45,    1,  290,    2, 0x0a /* Public */,
      46,    1,  293,    2, 0x0a /* Public */,
      47,    1,  296,    2, 0x0a /* Public */,
      49,    1,  299,    2, 0x0a /* Public */,
      51,    1,  302,    2, 0x0a /* Public */,
      52,    1,  305,    2, 0x0a /* Public */,
      54,    1,  308,    2, 0x0a /* Public */,
      55,    1,  311,    2, 0x0a /* Public */,
      56,    1,  314,    2, 0x0a /* Public */,
      57,    1,  317,    2, 0x08 /* Private */,
      59,    1,  320,    2, 0x08 /* Private */,
      61,    1,  323,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong,    7,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, 0x80000000 | 14, QMetaType::QString,   15,   16,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariant,   18,   19,
    QMetaType::Void, QMetaType::QSize,    2,
    QMetaType::Void, QMetaType::QString,   22,
    QMetaType::Void, QMetaType::QString,   22,
    QMetaType::Void, QMetaType::QString,   25,
    QMetaType::Void, QMetaType::QReal,   27,
    QMetaType::Void, QMetaType::Int,   27,
    QMetaType::Void, QMetaType::Int,   27,
    QMetaType::Void, QMetaType::Int,   27,
    QMetaType::Void, QMetaType::Int,   27,
    QMetaType::Void, 0x80000000 | 33,   34,
    QMetaType::Void, 0x80000000 | 33,   34,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariant,   18,   19,
    QMetaType::Void, QMetaType::QSize,   43,
    QMetaType::Void, QMetaType::QString,   22,
    QMetaType::Void, QMetaType::QString,   22,
    QMetaType::Void, QMetaType::QString,   25,
    QMetaType::Void, QMetaType::QReal,   48,
    QMetaType::Void, QMetaType::Int,   50,
    QMetaType::Void, QMetaType::Int,   50,
    QMetaType::Void, QMetaType::Int,   53,
    QMetaType::Void, QMetaType::Int,   50,
    QMetaType::Void, 0x80000000 | 33,   34,
    QMetaType::Void, 0x80000000 | 33,   34,
    QMetaType::Void, 0x80000000 | 58,    2,
    QMetaType::Void, 0x80000000 | 60,    2,
    QMetaType::Void, QMetaType::QUrl,    2,

 // properties: name, type, flags
      62, 0x80000000 | 63, 0x0049510b,
      64, 0x80000000 | 65, 0x00495009,
      66, QMetaType::QString, 0x00495103,
      43, QMetaType::QSize, 0x00495003,
      48, QMetaType::QReal, 0x00495103,
      67, QMetaType::Int, 0x00495103,
      68, 0x80000000 | 33, 0x0049510b,
      69, QMetaType::QString, 0x00495103,
      70, QMetaType::Int, 0x00495103,
      71, QMetaType::Int, 0x00495103,
      72, QMetaType::Int, 0x00495103,
      73, 0x80000000 | 33, 0x0049510b,
      74, QMetaType::QString, 0x00495103,
       7, QMetaType::LongLong, 0x00495001,
      75, QMetaType::QString, 0x00495103,
      76, QMetaType::QString, 0x00495001,
       9, QMetaType::Bool, 0x00495103,
      16, QMetaType::QString, 0x00495001,
      15, 0x80000000 | 77, 0x00495009,

 // properties: notify_signal_id
       0,
       1,
      10,
       8,
      12,
      13,
      18,
       9,
      14,
      15,
      16,
      17,
      11,
       2,
       4,
       5,
       3,
       6,
       6,

 // enums: name, alias, flags, count, data
      63,   63, 0x0,    2,  422,
      65,   65, 0x0,    8,  426,
      33,   33, 0x0,    3,  442,
      77,   77, 0x0,    4,  448,

 // enum data: key, value
      78, uint(QDeclarativeCameraRecorder::StoppedState),
      79, uint(QDeclarativeCameraRecorder::RecordingState),
      80, uint(QDeclarativeCameraRecorder::UnavailableStatus),
      81, uint(QDeclarativeCameraRecorder::UnloadedStatus),
      82, uint(QDeclarativeCameraRecorder::LoadingStatus),
      83, uint(QDeclarativeCameraRecorder::LoadedStatus),
      84, uint(QDeclarativeCameraRecorder::StartingStatus),
      85, uint(QDeclarativeCameraRecorder::RecordingStatus),
      86, uint(QDeclarativeCameraRecorder::PausedStatus),
      87, uint(QDeclarativeCameraRecorder::FinalizingStatus),
      88, uint(QDeclarativeCameraRecorder::ConstantQualityEncoding),
      89, uint(QDeclarativeCameraRecorder::ConstantBitRateEncoding),
      90, uint(QDeclarativeCameraRecorder::AverageBitRateEncoding),
      91, uint(QDeclarativeCameraRecorder::NoError),
      92, uint(QDeclarativeCameraRecorder::ResourceError),
      93, uint(QDeclarativeCameraRecorder::FormatError),
      94, uint(QDeclarativeCameraRecorder::OutOfSpaceError),

       0        // eod
};

void QDeclarativeCameraRecorder::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeclarativeCameraRecorder *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->recorderStateChanged((*reinterpret_cast< QDeclarativeCameraRecorder::RecorderState(*)>(_a[1]))); break;
        case 1: _t->recorderStatusChanged(); break;
        case 2: _t->durationChanged((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 3: _t->mutedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->outputLocationChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->actualLocationChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->error((*reinterpret_cast< QDeclarativeCameraRecorder::Error(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 7: _t->metaDataChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 8: _t->captureResolutionChanged((*reinterpret_cast< const QSize(*)>(_a[1]))); break;
        case 9: _t->audioCodecChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->videoCodecChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->mediaContainerChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->frameRateChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 13: _t->videoBitRateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->audioBitRateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->audioChannelsChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->audioSampleRateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->audioEncodingModeChanged((*reinterpret_cast< EncodingMode(*)>(_a[1]))); break;
        case 18: _t->videoEncodingModeChanged((*reinterpret_cast< EncodingMode(*)>(_a[1]))); break;
        case 19: _t->setOutputLocation((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 20: _t->record(); break;
        case 21: _t->stop(); break;
        case 22: _t->setRecorderState((*reinterpret_cast< QDeclarativeCameraRecorder::RecorderState(*)>(_a[1]))); break;
        case 23: _t->setMuted((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->setMetadata((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 25: _t->setCaptureResolution((*reinterpret_cast< const QSize(*)>(_a[1]))); break;
        case 26: _t->setAudioCodec((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 27: _t->setVideoCodec((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 28: _t->setMediaContainer((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 29: _t->setFrameRate((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 30: _t->setVideoBitRate((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 31: _t->setAudioBitRate((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 32: _t->setAudioChannels((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 33: _t->setAudioSampleRate((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 34: _t->setVideoEncodingMode((*reinterpret_cast< EncodingMode(*)>(_a[1]))); break;
        case 35: _t->setAudioEncodingMode((*reinterpret_cast< EncodingMode(*)>(_a[1]))); break;
        case 36: _t->updateRecorderState((*reinterpret_cast< QMediaRecorder::State(*)>(_a[1]))); break;
        case 37: _t->updateRecorderError((*reinterpret_cast< QMediaRecorder::Error(*)>(_a[1]))); break;
        case 38: _t->updateActualLocation((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaRecorder::State >(); break;
            }
            break;
        case 37:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaRecorder::Error >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeclarativeCameraRecorder::*)(QDeclarativeCameraRecorder::RecorderState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::recorderStateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::recorderStatusChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::durationChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::mutedChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::outputLocationChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::actualLocationChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(QDeclarativeCameraRecorder::Error , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::error)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(const QString & , const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::metaDataChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(const QSize & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::captureResolutionChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::audioCodecChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::videoCodecChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::mediaContainerChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::frameRateChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::videoBitRateChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::audioBitRateChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::audioChannelsChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::audioSampleRateChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(EncodingMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::audioEncodingModeChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QDeclarativeCameraRecorder::*)(EncodingMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeCameraRecorder::videoEncodingModeChanged)) {
                *result = 18;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeCameraRecorder *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< RecorderState*>(_v) = _t->recorderState(); break;
        case 1: *reinterpret_cast< RecorderStatus*>(_v) = _t->recorderStatus(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->videoCodec(); break;
        case 3: *reinterpret_cast< QSize*>(_v) = _t->captureResolution(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->frameRate(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->videoBitRate(); break;
        case 6: *reinterpret_cast< EncodingMode*>(_v) = _t->videoEncodingMode(); break;
        case 7: *reinterpret_cast< QString*>(_v) = _t->audioCodec(); break;
        case 8: *reinterpret_cast< int*>(_v) = _t->audioBitRate(); break;
        case 9: *reinterpret_cast< int*>(_v) = _t->audioChannels(); break;
        case 10: *reinterpret_cast< int*>(_v) = _t->audioSampleRate(); break;
        case 11: *reinterpret_cast< EncodingMode*>(_v) = _t->audioEncodingMode(); break;
        case 12: *reinterpret_cast< QString*>(_v) = _t->mediaContainer(); break;
        case 13: *reinterpret_cast< qint64*>(_v) = _t->duration(); break;
        case 14: *reinterpret_cast< QString*>(_v) = _t->outputLocation(); break;
        case 15: *reinterpret_cast< QString*>(_v) = _t->actualLocation(); break;
        case 16: *reinterpret_cast< bool*>(_v) = _t->isMuted(); break;
        case 17: *reinterpret_cast< QString*>(_v) = _t->errorString(); break;
        case 18: *reinterpret_cast< Error*>(_v) = _t->errorCode(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QDeclarativeCameraRecorder *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRecorderState(*reinterpret_cast< RecorderState*>(_v)); break;
        case 2: _t->setVideoCodec(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setCaptureResolution(*reinterpret_cast< QSize*>(_v)); break;
        case 4: _t->setFrameRate(*reinterpret_cast< qreal*>(_v)); break;
        case 5: _t->setVideoBitRate(*reinterpret_cast< int*>(_v)); break;
        case 6: _t->setVideoEncodingMode(*reinterpret_cast< EncodingMode*>(_v)); break;
        case 7: _t->setAudioCodec(*reinterpret_cast< QString*>(_v)); break;
        case 8: _t->setAudioBitRate(*reinterpret_cast< int*>(_v)); break;
        case 9: _t->setAudioChannels(*reinterpret_cast< int*>(_v)); break;
        case 10: _t->setAudioSampleRate(*reinterpret_cast< int*>(_v)); break;
        case 11: _t->setAudioEncodingMode(*reinterpret_cast< EncodingMode*>(_v)); break;
        case 12: _t->setMediaContainer(*reinterpret_cast< QString*>(_v)); break;
        case 14: _t->setOutputLocation(*reinterpret_cast< QString*>(_v)); break;
        case 16: _t->setMuted(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QDeclarativeCameraRecorder::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeCameraRecorder.data,
    qt_meta_data_QDeclarativeCameraRecorder,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeclarativeCameraRecorder::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeCameraRecorder::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeCameraRecorder.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeCameraRecorder::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 19;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeclarativeCameraRecorder::recorderStateChanged(QDeclarativeCameraRecorder::RecorderState _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QDeclarativeCameraRecorder::recorderStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeCameraRecorder::durationChanged(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QDeclarativeCameraRecorder::mutedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QDeclarativeCameraRecorder::outputLocationChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QDeclarativeCameraRecorder::actualLocationChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QDeclarativeCameraRecorder::error(QDeclarativeCameraRecorder::Error _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QDeclarativeCameraRecorder::metaDataChanged(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QDeclarativeCameraRecorder::captureResolutionChanged(const QSize & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QDeclarativeCameraRecorder::audioCodecChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QDeclarativeCameraRecorder::videoCodecChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QDeclarativeCameraRecorder::mediaContainerChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QDeclarativeCameraRecorder::frameRateChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QDeclarativeCameraRecorder::videoBitRateChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QDeclarativeCameraRecorder::audioBitRateChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QDeclarativeCameraRecorder::audioChannelsChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QDeclarativeCameraRecorder::audioSampleRateChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QDeclarativeCameraRecorder::audioEncodingModeChanged(EncodingMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QDeclarativeCameraRecorder::videoEncodingModeChanged(EncodingMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
