/****************************************************************************
** Meta object code from reading C++ file 'qsoundeffect_pulse_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/multimedia/audio/qsoundeffect_pulse_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsoundeffect_pulse_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QSoundEffectPrivate_t {
    QByteArrayData data[24];
    char stringdata0[297];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QSoundEffectPrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QSoundEffectPrivate_t qt_meta_stringdata_QSoundEffectPrivate = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QSoundEffectPrivate"
QT_MOC_LITERAL(1, 20, 21), // "loopsRemainingChanged"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 13), // "volumeChanged"
QT_MOC_LITERAL(4, 57, 12), // "mutedChanged"
QT_MOC_LITERAL(5, 70, 13), // "loadedChanged"
QT_MOC_LITERAL(6, 84, 14), // "playingChanged"
QT_MOC_LITERAL(7, 99, 13), // "statusChanged"
QT_MOC_LITERAL(8, 113, 15), // "categoryChanged"
QT_MOC_LITERAL(9, 129, 4), // "play"
QT_MOC_LITERAL(10, 134, 4), // "stop"
QT_MOC_LITERAL(11, 139, 12), // "decoderError"
QT_MOC_LITERAL(12, 152, 11), // "sampleReady"
QT_MOC_LITERAL(13, 164, 12), // "uploadSample"
QT_MOC_LITERAL(14, 177, 12), // "contextReady"
QT_MOC_LITERAL(15, 190, 13), // "contextFailed"
QT_MOC_LITERAL(16, 204, 8), // "underRun"
QT_MOC_LITERAL(17, 213, 7), // "prepare"
QT_MOC_LITERAL(18, 221, 11), // "streamReady"
QT_MOC_LITERAL(19, 233, 13), // "emptyComplete"
QT_MOC_LITERAL(20, 247, 6), // "stream"
QT_MOC_LITERAL(21, 254, 6), // "reload"
QT_MOC_LITERAL(22, 261, 25), // "handleAvailabilityChanged"
QT_MOC_LITERAL(23, 287, 9) // "available"

    },
    "QSoundEffectPrivate\0loopsRemainingChanged\0"
    "\0volumeChanged\0mutedChanged\0loadedChanged\0"
    "playingChanged\0statusChanged\0"
    "categoryChanged\0play\0stop\0decoderError\0"
    "sampleReady\0uploadSample\0contextReady\0"
    "contextFailed\0underRun\0prepare\0"
    "streamReady\0emptyComplete\0stream\0"
    "reload\0handleAvailabilityChanged\0"
    "available"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QSoundEffectPrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  109,    2, 0x06 /* Public */,
       3,    0,  110,    2, 0x06 /* Public */,
       4,    0,  111,    2, 0x06 /* Public */,
       5,    0,  112,    2, 0x06 /* Public */,
       6,    0,  113,    2, 0x06 /* Public */,
       7,    0,  114,    2, 0x06 /* Public */,
       8,    0,  115,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,  116,    2, 0x0a /* Public */,
      10,    0,  117,    2, 0x0a /* Public */,
      11,    0,  118,    2, 0x08 /* Private */,
      12,    0,  119,    2, 0x08 /* Private */,
      13,    0,  120,    2, 0x08 /* Private */,
      14,    0,  121,    2, 0x08 /* Private */,
      15,    0,  122,    2, 0x08 /* Private */,
      16,    0,  123,    2, 0x08 /* Private */,
      17,    0,  124,    2, 0x08 /* Private */,
      18,    0,  125,    2, 0x08 /* Private */,
      19,    2,  126,    2, 0x08 /* Private */,
      22,    1,  131,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::VoidStar, QMetaType::Bool,   20,   21,
    QMetaType::Void, QMetaType::Bool,   23,

       0        // eod
};

void QSoundEffectPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QSoundEffectPrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->loopsRemainingChanged(); break;
        case 1: _t->volumeChanged(); break;
        case 2: _t->mutedChanged(); break;
        case 3: _t->loadedChanged(); break;
        case 4: _t->playingChanged(); break;
        case 5: _t->statusChanged(); break;
        case 6: _t->categoryChanged(); break;
        case 7: _t->play(); break;
        case 8: _t->stop(); break;
        case 9: _t->decoderError(); break;
        case 10: _t->sampleReady(); break;
        case 11: _t->uploadSample(); break;
        case 12: _t->contextReady(); break;
        case 13: _t->contextFailed(); break;
        case 14: _t->underRun(); break;
        case 15: _t->prepare(); break;
        case 16: _t->streamReady(); break;
        case 17: _t->emptyComplete((*reinterpret_cast< void*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 18: _t->handleAvailabilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QSoundEffectPrivate::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffectPrivate::loopsRemainingChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QSoundEffectPrivate::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffectPrivate::volumeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QSoundEffectPrivate::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffectPrivate::mutedChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QSoundEffectPrivate::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffectPrivate::loadedChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QSoundEffectPrivate::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffectPrivate::playingChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QSoundEffectPrivate::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffectPrivate::statusChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QSoundEffectPrivate::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffectPrivate::categoryChanged)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QSoundEffectPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QSoundEffectPrivate.data,
    qt_meta_data_QSoundEffectPrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QSoundEffectPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSoundEffectPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QSoundEffectPrivate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QSoundEffectPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void QSoundEffectPrivate::loopsRemainingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QSoundEffectPrivate::volumeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QSoundEffectPrivate::mutedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QSoundEffectPrivate::loadedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QSoundEffectPrivate::playingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QSoundEffectPrivate::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QSoundEffectPrivate::categoryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
